#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 1A - Enoncé 24 novembre 2020
# 
# Correction de l'examen du 24 novembre 2020.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Exercice 1 : guérison
# 
# On commence par générer des données artificielles à partir de véritables données.

# In[2]:


import pandas 
df = pandas.read_csv("https://www.data.gouv.fr/en/datasets/r/63352e38-d353-4b54-bfd1-f1b3ee1cabd7", sep=";")
gr = df[["jour", "rad", "dc"]].groupby(["jour"]).sum()
gr.head()


# In[3]:


diff = gr.diff().reset_index(drop=False)
diff.head()


# On convertit la date en jour de l'année puis on simule un loi exponentielle de paramètre 14 pour avoir la date de sortie.

# In[4]:


import numpy.random as rnd


def donnees_artificielles(hosp, mu=14, nu=21):
    dt = pandas.to_datetime(hosp['jour'])
    res = []
    for i in range(hosp.shape[0]):
        date = dt[i].dayofyear
        h = hosp.iloc[i, 1]
        delay = rnd.exponential(mu, int(h))
        for j in range(delay.shape[0]):
            res.append([date - int(delay[j]), date, 1])
        h = hosp.iloc[i, 2]
        delay = rnd.exponential(nu, int(h))
        for j in range(delay.shape[0]):
            res.append([date - int(delay[j]), date , 0])
    return pandas.DataFrame(res, columns=["entree", "sortie", "issue"])
            
        
data = donnees_artificielles(diff[1:].reset_index(drop=True))
data.head()


# In[5]:


data.to_csv("examen2021.csv", index=False)


# ### Q1
# 
# On récupère les données.

# In[6]:


import pandas
df = pandas.read_csv("http://www.xavierdupre.fr/enseignement/complements/examen2021.zip")
df.head()


# ### Q2 : durée de guérison

# In[7]:


duree = df['sortie'] - df['entree']
duree = duree.values  # conversion en numpy
issue = df['issue'].values
duree[:5], issue[:5]


# ### Q3 : estimateur Kaplan-Meier (1)

# In[8]:


t = 10
nt = duree[(duree >= t)].shape[0]
dt = duree[(duree == t) & (issue == 0)].shape[0]
st = 1. - dt / nt
st


# ### Q4 : courbe de Kaplan-Meier

# In[9]:


T = [0]
St = [1.]
for t in range(0, 150):
    nt = duree[(duree >= t)].shape[0]
    dt = duree[(duree == t) & (issue == 0)].shape[0]
    st = 1. - dt / nt
    T.append(t)
    St.append(st * St[-1])


# ### Q5 : graphe

# In[10]:


import matplotlib.pyplot as plt
fig, ax = plt.subplots(1, 1, figsize=(4, 4))
ax.plot(T, St);


# In[11]:


import numpy
from lifelines import KaplanMeierFitter
fig, ax = plt.subplots(1, 1, figsize=(10, 4))
ax.plot(T, St, label="custom", lw=10)

kmf = KaplanMeierFitter()
kmf.fit(duree, (issue == 0).astype(numpy.int32))
kmf.plot(ax=ax)
ax.legend();


# ### Q6 : application aux données publiques
# 
# Les données accessibles librement sur le portail [data.gouv.fr](https://www.data.gouv.fr/fr/) recensent les entrées et les sorties des personnes sans relier une entrée et une sortie spécifique. Si N est personnes sont sorties guéries, on ne sait pas quand elles sont entrées. Donc le calcul ci-dessus n'est pas possible.

# ## Exercice 2

# ### Q1 : t + 1

# In[12]:


import numpy

N = 10
M = numpy.zeros((N, N))
M[4, 5] = 1
M


# In[13]:


def propagation(M):
    M2 = M.copy()
    M2[1:, :] = numpy.maximum(M2[1:, :], M[:-1, :])
    M2[:-1, :] = numpy.maximum(M2[:-1, :], M[1:, :])
    M2[:, 1:] = numpy.maximum(M2[:, 1:], M[:, :-1])
    M2[:, :-1] = numpy.maximum(M2[:, :-1], M[:, 1:])
    return M2
    
propagation(M.copy())


# ### Q2 : après T itération

# In[14]:


def propagation_n(M, t):
    for i in range(t):
        M = propagation(M)
    return M

propagation_n(M, 3)


# ### Q3 : vaccin

# In[15]:


p = 0.3
vaccine = (numpy.random.rand(N, N) <= p).astype(numpy.int32)
vaccine


# In[16]:


def propagation_vaccine(M, vaccine):
    M2 = M.copy()
    M2[1:, :] = numpy.maximum(M2[1:, :], M[:-1, :])
    M2[:-1, :] = numpy.maximum(M2[:-1, :], M[1:, :])
    M2[:, 1:] = numpy.maximum(M2[:, 1:], M[:, :-1])
    M2[:, :-1] = numpy.maximum(M2[:, :-1], M[:, 1:])
    M2 = numpy.minimum(M2, 1 - vaccine)
    return M2

vaccine[4, 5] = 0
propagation_vaccine(M, vaccine)


# ### Q4 : après T heures

# In[17]:


def propagation_n_vaccine(M, t, vaccine):
    for i in range(t):
        M = propagation_vaccine(M, vaccine)
    return M

propagation_n_vaccine(M, 3, vaccine)


# ### Q5 : variation

# In[18]:


import pandas

res = []
for p in [0.7, 0.8, 0.85, 0.86, 0.87, 0.88, 0.89, 0.9, 0.94, 0.95, 0.96, 0.97, 0.98, 0.99, 1.0]:
    cont = []
    for test in range(0, 10):
        vaccine = (numpy.random.rand(N, N) <= p).astype(numpy.int32)
        M[4, 5] = 1
        vaccine[4, 5] = 0
        M = propagation_n_vaccine(M, 3, vaccine)
        contamine = M.ravel().sum()
        cont.append(contamine)
    cont = numpy.array(cont)
    res.append(dict(p=p, prop=cont.mean()))

df = pandas.DataFrame(res)
df


# In[19]:


ax = df.plot(x="p", y="prop")
ax.plot([0.7, 1.], [2, 2], "--");


# Dans cette configuration, en supposant qu'un seul élève est contaminé, il faudrait vacciner à plus de 85% pour avoir une chance de ne pas avoir une nouvelle contamination.
# 
# Ce résultat ne change pas si la taille de la matrice change. Il change en revanche en fonction du nombre de tirages, ici 10. Pour n'avoir aucune propagation, il faut entourer la personne contaminé de 4 personnes vaccinées. $p$ est la probabilité pour une personné d'être vaccinée (ou immunisée si la vaccination est faite sur 100% des personnes). $p^4$ est la probabilité d'avoir 4 personnes vaccinées. $q=1-p^4$ est la probabilité qu'une personne ne soient pas immunisées parmi les 4. $1 - (1-q)^{10}$ est la probabilité que 10 cours de 3h se passent bien sans contamination.

# In[20]:


p = 0.9
p4 = 1 - (1 - p) **4
p4


# In[21]:


df2 = df.copy()
df2["P4**100"] = 1 - df2["p"] ** 40


# In[22]:


import matplotlib.pyplot as plt
fig, ax = plt.subplots(1, 2, figsize=(10, 4))
df2.plot(x="p", y="prop", ax=ax[0])
ax[0].plot([0.7, 1.], [2, 2], "--")
df2.plot(x="p", y="P4**100", ax=ax[1]);


# In[23]:





# In[24]: